<?php

require_once 'tools/class.type.php' ;

/** Manage MySQL access, and provide basics methods.
  * Must be derivate to implement basic method for a record
  */
abstract class MyType
{
	const SQL_FIND_FIRST = "select id from %s where id = ? order by %s limit 1" ;
	const SQL_SELECT_BY_ID = "select id, title, content from %s where id = ?" ;
	const SQL_SELECT_BY_NAME = "select id, title, content from %s where title = ?" ;
	const SQL_INSERT = "insert %s (id, title, content) values (?, ?, ?, ?, ?)" ;
	const SQL_REPLACE = "replace %s (id, title, content) values values (?, ?, ?, ?, ?)" ;
	const SQL_UPDATE = "update %s set title = ?, content = ? where id = ?" ;
	const SQL_DELETE = "delete from %s where id = ?" ;

	const SQL_FIND_NAME = "select id, title, content from %s where title = ?" ;

	abstract protected function setThingFromSQLStmt($stmt) ;
	abstract protected function setSQLStmtFromThing($stmt) ;
	abstract protected function setThingFromStdObject($obj) ;
	abstract protected function setThingFromArray($obj) ;

	/**
	  * @param object|Type $attrs Data that permit to fetch informations
	  */
	public function __construct($attrs = null)
	{
		if(is_null($attrs))
			return ;

		if(is_object($attrs))
			$this->setThingFromStdObject($attrs) ;

		if(is_array($attrs))
			$this->setThingFromArray($attrs) ;
	}

	abstract protected function getSQLTemplate($method);
	protected function getSQLStmt($method)
	{
		$sqlTemplate = $this->getSQLTemplate($method) ;
		switch($method)
		{
			case 'FIND_FIRST' :
				$sql = sprintf
					( $sqlTemplate
					, $this->getTableName()
					, $this->getOrderedField()) ;
				break ;
			default:
				$sql = sprintf($sqlTemplate, $this->getTableName()) ;
		}

		$myStmt = $this->_handler->stmt_init() ;
		if(!$myStmt->prepare($sql)) trigger_error($myStmt->error) ;

		return $myStmt ;
	}

	protected function getOrderedField() { return 'id' ; }
	
	public function getFirstId()
	{
		$sqlStmt = $this->getSQLStmt('FIND_FIRST') ;
		$sqlStmt->bind_result($id) ;
		$sqlStmt->execute() ; 
		$sqlStmt->store_result() ; 
		$sqlStmt->fetch() ;
		$sqlStmt->free_result() ; 
		$sqlStmt->close() ;

		return $this->_thing->id = $id ;
	}

	/** Use id to fetch information about adventure
	  * @param integer $id Adventure Id. If $id is null, find with $_thing->id
	  */
	public function select($id = null)
	{
		$this->_thing->id = is_null($id) ? $this->_thing->id : $id ;
		$sqlStmt = $this->getSQLStmt('SELECT_BY_ID') ;
		$sqlStmt->bind_param('i', $this->_thing->id) ;

		return $this->setThingFromSQLStmt($sqlStmt) ;
	}

	/** Use title to fetch information about adventure
	  * @param integer $title Adventure's title. If $title is null, find with $_thing->title
	  */
	public function findByTitle($title = null)
	{
		$title = $this->_handler->real_escape_string
			(is_null($title) ? $this->_thing->title : $title) ;
		$sqlStmt = $this->getSQLStmt('SELECT_BY_NAME') ;
		$sqlStmt->bind_param('s', $title) ;
		$sqlStmt->execute() ;

		return $this->setThingFromSQLStmt($sqlStmt) ;
	}

	public function findByName($name)
	{
		return $this->findByTitle($name) ;
	}

	public function insert()
	{
		$sqlStmt = $this->getSQLStmt('INSERT') ;
		return $this->setSQLStmtFromThing($sqlStmt) ;
	}
	
	public function update()
	{
		$sqlStmt = $this->getSQLStmt('UPDATE') ;
		return $this->setSQLStmtFromThing($sqlStmt) ;
	}
	
	public function replace(MyType $new)
	{
		$sqlStmt = $this->getSQLStmt('REPLACE') ;
		return $this->setSQLStmtFromThing($sqlStmt) ;
	}
	
	public function add()
	{
		return $this->insert() ;
	}
	
	public function delete()
	{
		$sqlStmt = $this->getSQLStmt('DELETE') ;
		$sqlStmt->bind_param('i', $this->_thing->id) ;
		$ret = ($sqlStmt->execute() and $sqlStmt->affected_rows > 0) ;

		$sqlStmt->close() ;

		return $ret ;
	}


	public function connect($handler)
	{
		$this->_handler = $handler ;
	}
	
	/** Querying SQL base
	  *
	  * @param string $sql Query to execute
	  * @return array|object|resource|null|false
	  */
	protected function query($sql) 
	{
#		echo "<p>Query : <code>$sql</code></p>" ;

		$res = $this->_handler->query($sql) ;
		if($res and $res instanceof MySQLi_Result)
		{
			$obj = $res->fetch_object() ;
			return $obj ;
		}
		else
			return $res ;

	}

	/** 
	  * @param string $sql
	  * @return boolean|MySQLiRes
	  */
	protected function exec($sql)
	{
		return $this->_handler->exec($sql) ;
	}

	/** @return MySQLi $handler */
	protected function getHandler()
	{
		return $this->_handler ;
	}

	/** @param MySQLi $handler */
	protected function setHandler(MySQLi $handler)
	{
		$this->_handler = $handler ;
	}

	protected function getTableName()
	{
		return strlen($this->_myTableNamePrefix) > 0
			? "{$this->_myTableNamePrefix}_{$this->_myTableName}"
			: $this->_myTableName ;
	}

	/** Container for ressource to MySQL con */
	private $_handler = null ;
	/** Container for business object managed */
	protected $_thing = null ;

	protected $_myThingClass = '' ;
	protected $_myTableName = '' ;
	protected $_myTableNamePrefix = '' ;
}

abstract class MyCollection
{
	// abstract protected function setThingCollectionFromCollection($collection) ;

	public function __construct($collection)
	{
		$this->setThingCollectionFromCollection($collection) ;
	}

	/** @param MySQLi $handler */
	protected function setHandler(MySQLi $handler)
	{
		$this->_handler = $handler ;
	}

	/** Container for ressource to MySQL con */
	private $_handler = null ;
	/** Container for business object manage */
	protected $_thingCollection = null ;

	protected $_myThingCollectionClass = '' ;
}

?>
